@@ -3,7 +3,7 @@ use strict;
 use warnings;
 package Email::MIME;
 # ABSTRACT: easy MIME message handling
-$Email::MIME::VERSION = '1.936';
+$Email::MIME::VERSION = '1.940';
 use Email::Simple 2.206; # header_raw
 use parent qw(Email::Simple);
 
@@ -16,7 +16,7 @@ use Email::MIME::Encodings 1.314;
 use Email::MIME::Header;
 use Email::MIME::Modifier;
 use Encode 1.9801 ();
-use Scalar::Util qw(reftype);
+use Scalar::Util qw(reftype weaken);
 
 #pod =head1 SYNOPSIS
 #pod
@@ -83,7 +83,7 @@ use Scalar::Util qw(reftype);
 #pod   $_->encoding_set( 'base64' ) for $email->parts;
 #pod
 #pod   # Quick multipart creation
-#pod   my $quicky = Email::MIME->create(
+#pod   my $email = Email::MIME->create(
 #pod       header_str => [
 #pod           From => 'my@address',
 #pod           To   => 'your@address',
@@ -702,8 +702,14 @@ sub parts_set {
       $body .= $part->as_string;
     }
     $body .= "$self->{mycrlf}--$bound--$self->{mycrlf}";
-    @{$ct_header}{qw[type subtype]} = qw[multipart mixed]
-      unless grep { $ct_header->{type} eq $_ } qw[multipart message];
+
+    unless (grep { $ct_header->{type} eq $_ } qw[multipart message]) {
+      if (scalar $self->header('Content-Type')) {
+        Carp::carp("replacing non-multipart type ($ct_header->{type}/$ct_header->{subtype}) with multipart/mixed");
+      }
+      @{$ct_header}{qw[type subtype]} = qw[multipart mixed];
+    }
+
     $self->encoding_set('7bit');
     delete $ct_header->{attributes}{charset};
   } elsif (@$parts == 1) {  # setup singlepart
@@ -764,8 +770,8 @@ sub walk_parts {
 
   my %changed;
 
-  my $walk;
-  $walk = sub {
+  my $walk_weak;
+  my $walk = sub {
     my ($part) = @_;
     $callback->($part);
 
@@ -775,7 +781,7 @@ sub walk_parts {
 
       for my $part (@orig_subparts) {
         my $str = $part->as_string;
-        next unless my $new = $walk->($part);
+        next unless my $new = $walk_weak->($part);
         $differ = 1 if $str ne $new->as_string;
         push @subparts, $new;
       }
@@ -794,6 +800,9 @@ sub walk_parts {
     return $part;
   };
 
+  $walk_weak = $walk;
+  weaken $walk_weak;
+
   my $rv = $walk->($self);
 
   undef $walk;
@@ -857,7 +866,7 @@ Email::MIME - easy MIME message handling
 
 =head1 VERSION
 
-version 1.936
+version 1.940
 
 =head1 SYNOPSIS
 
@@ -924,7 +933,7 @@ by all means keep reading.
   $_->encoding_set( 'base64' ) for $email->parts;
 
   # Quick multipart creation
-  my $quicky = Email::MIME->create(
+  my $email = Email::MIME->create(
       header_str => [
           From => 'my@address',
           To   => 'your@address',
@@ -1233,7 +1242,7 @@ Simon Cozens <simon@cpan.org>
 
 =head1 CONTRIBUTORS
 
-=for stopwords Alex Vandiver Arthur Axel 'fREW' Schmidt Brian Cassidy David Steinbrunner Dotan Dimet Geraint Edwards Jesse Luehrs Kurt Anderson Lance A. Brown Matthew Horsfall (alh) memememomo Shawn Sorichetti Tomohiro Hosaka
+=for stopwords Alex Vandiver Arthur Axel 'fREW' Schmidt Brian Cassidy David Steinbrunner Dotan Dimet Geraint Edwards Jesse Luehrs Kurt Anderson Lance A. Brown Matthew Horsfall (alh) memememomo Pali Shawn Sorichetti Tomohiro Hosaka
 
 =over 4
 
@@ -1283,6 +1292,10 @@ memememomo <memememomo@gmail.com>
 
 =item *
 
+Pali <pali@cpan.org>
+
+=item *
+
 Shawn Sorichetti <ssoriche@coloredblocks.com>
 
 =item *
