@@ -2,7 +2,7 @@ use strict;
 use warnings;
 package Email::MIME::Header;
 # ABSTRACT: the header of a MIME message
-$Email::MIME::Header::VERSION = '1.934';
+$Email::MIME::Header::VERSION = '1.940';
 use parent 'Email::Simple::Header';
 
 use Email::MIME::Encode;
@@ -22,7 +22,8 @@ use Encode 1.9801;
 #pod encoded header.  Thus, C<header_set> round-trips with C<header_raw>,
 #pod not C<header>!  Be sure to properly encode your headers with
 #pod C<Encode::encode('MIME-Header', $value)> before passing them to
-#pod C<header_set>.
+#pod C<header_set>.  And be sure to use minimal version 2.83 of Encode
+#pod module due to L<bugs in MIME-Header|Encode::MIME::Header/BUGS>.
 #pod
 #pod Alternately, if you have Unicode (character) strings to set in headers, use the
 #pod C<header_str_set> method.
@@ -38,19 +39,18 @@ sub header_str {
   my @header = $wanta ? $self->header_raw(@_)
                       : scalar $self->header_raw(@_);
 
-  local $@;
   foreach my $header (@header) {
     next unless defined $header;
     next unless $header =~ /=\?/;
 
-    _maybe_decode(\$header);
+    _maybe_decode($_[0], \$header);
   }
   return $wanta ? @header : $header[0];
 }
 
 sub header {
-  my ($self, $name) = @_;
-  return $self->header_str($name);
+  my $self = shift;
+  return $self->header_str(@_);
 }
 
 sub header_str_set {
@@ -68,21 +68,15 @@ sub header_str_pairs {
 
   my @pairs = $self->header_pairs;
   for (grep { $_ % 2 } (1 .. $#pairs)) {
-    _maybe_decode(\$pairs[$_]);
+    _maybe_decode($pairs[$_-1], \$pairs[$_]);
   }
 
   return @pairs;
 }
 
 sub _maybe_decode {
-  my ($str_ref) = @_;
-
-  # The eval is to cope with unknown encodings, like Latin-62, or other
-  # nonsense that gets put in there by spammers and weirdos
-  # -- rjbs, 2014-12-04
-  my $new;
-  $$str_ref = $new
-    if eval { $new = Encode::decode("MIME-Header", $$str_ref); 1 };
+  my ($name, $str_ref) = @_;
+  $$str_ref = Email::MIME::Encode::maybe_mime_decode_header($name, $$str_ref);
   return;
 }
 
@@ -100,7 +94,7 @@ Email::MIME::Header - the header of a MIME message
 
 =head1 VERSION
 
-version 1.934
+version 1.940
 
 =head1 DESCRIPTION
 
@@ -127,7 +121,8 @@ Note that C<header_set> does not do encoding for you, and expects an
 encoded header.  Thus, C<header_set> round-trips with C<header_raw>,
 not C<header>!  Be sure to properly encode your headers with
 C<Encode::encode('MIME-Header', $value)> before passing them to
-C<header_set>.
+C<header_set>.  And be sure to use minimal version 2.83 of Encode
+module due to L<bugs in MIME-Header|Encode::MIME::Header/BUGS>.
 
 Alternately, if you have Unicode (character) strings to set in headers, use the
 C<header_str_set> method.
